@@ -1,7 +1,7 @@
 /*
  * wee-hook.c - WeeChat hooks management
  *
- * Copyright (C) 2003-2022 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -31,12 +31,15 @@
 #include <errno.h>
 
 #include "weechat.h"
+#include "wee-debug.h"
 #include "wee-hook.h"
 #include "wee-hashtable.h"
 #include "wee-infolist.h"
 #include "wee-log.h"
 #include "wee-signal.h"
 #include "wee-string.h"
+#include "wee-sys.h"
+#include "wee-util.h"
 #include "../gui/gui-chat.h"
 #include "../plugins/plugin.h"
 
@@ -44,7 +47,7 @@
 char *hook_type_string[HOOK_NUM_TYPES] =
 { "command", "command_run", "timer", "fd", "process", "connect", "line",
   "print", "signal", "hsignal", "config", "completion", "modifier",
-  "info", "info_hashtable", "infolist", "hdata", "focus" };
+  "info", "info_hashtable", "infolist", "hdata", "focus", "url" };
 struct t_hook *weechat_hooks[HOOK_NUM_TYPES];     /* list of hooks          */
 struct t_hook *last_weechat_hook[HOOK_NUM_TYPES]; /* last hook              */
 int hooks_count[HOOK_NUM_TYPES];                  /* number of hooks        */
@@ -57,50 +60,98 @@ int hook_socketpair_ok = 0;            /* 1 if socketpair() is OK           */
 /* hook callbacks */
 t_callback_hook *hook_callback_add[HOOK_NUM_TYPES] =
 { NULL, NULL, NULL, &hook_fd_add_cb, NULL, NULL, NULL, NULL, NULL, NULL,
-  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 t_callback_hook *hook_callback_remove[HOOK_NUM_TYPES] =
 { NULL, NULL, NULL, &hook_fd_remove_cb, NULL, NULL, NULL, NULL, NULL, NULL,
-  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 t_callback_hook *hook_callback_free_data[HOOK_NUM_TYPES] =
-{ &hook_command_free_data, &hook_command_run_free_data,
-  &hook_timer_free_data, &hook_fd_free_data,
-  &hook_process_free_data, &hook_connect_free_data,
-  &hook_line_free_data, &hook_print_free_data,
-  &hook_signal_free_data, &hook_hsignal_free_data,
-  &hook_config_free_data, &hook_completion_free_data,
-  &hook_modifier_free_data, &hook_info_free_data,
-  &hook_info_hashtable_free_data, &hook_infolist_free_data,
-  &hook_hdata_free_data, &hook_focus_free_data };
+{
+    &hook_command_free_data,
+    &hook_command_run_free_data,
+    &hook_timer_free_data,
+    &hook_fd_free_data,
+    &hook_process_free_data,
+    &hook_connect_free_data,
+    &hook_line_free_data,
+    &hook_print_free_data,
+    &hook_signal_free_data,
+    &hook_hsignal_free_data,
+    &hook_config_free_data,
+    &hook_completion_free_data,
+    &hook_modifier_free_data,
+    &hook_info_free_data,
+    &hook_info_hashtable_free_data,
+    &hook_infolist_free_data,
+    &hook_hdata_free_data,
+    &hook_focus_free_data,
+    &hook_url_free_data,
+};
 t_callback_hook_get_desc *hook_callback_get_desc[HOOK_NUM_TYPES] =
-{ &hook_command_get_description, &hook_command_run_get_description,
-  &hook_timer_get_description, &hook_fd_get_description,
-  &hook_process_get_description, &hook_connect_get_description,
-  &hook_line_get_description, &hook_print_get_description,
-  &hook_signal_get_description, &hook_hsignal_get_description,
-  &hook_config_get_description, &hook_completion_get_description,
-  &hook_modifier_get_description, &hook_info_get_description,
-  &hook_info_hashtable_get_description, &hook_infolist_get_description,
-  &hook_hdata_get_description, &hook_focus_get_description };
+{
+    &hook_command_get_description,
+    &hook_command_run_get_description,
+    &hook_timer_get_description,
+    &hook_fd_get_description,
+    &hook_process_get_description,
+    &hook_connect_get_description,
+    &hook_line_get_description,
+    &hook_print_get_description,
+    &hook_signal_get_description,
+    &hook_hsignal_get_description,
+    &hook_config_get_description,
+    &hook_completion_get_description,
+    &hook_modifier_get_description,
+    &hook_info_get_description,
+    &hook_info_hashtable_get_description,
+    &hook_infolist_get_description,
+    &hook_hdata_get_description,
+    &hook_focus_get_description,
+    &hook_url_get_description,
+};
 t_callback_hook_infolist *hook_callback_add_to_infolist[HOOK_NUM_TYPES] =
-{ &hook_command_add_to_infolist, &hook_command_run_add_to_infolist,
-  &hook_timer_add_to_infolist, &hook_fd_add_to_infolist,
-  &hook_process_add_to_infolist, &hook_connect_add_to_infolist,
-  &hook_line_add_to_infolist, &hook_print_add_to_infolist,
-  &hook_signal_add_to_infolist, &hook_hsignal_add_to_infolist,
-  &hook_config_add_to_infolist, &hook_completion_add_to_infolist,
-  &hook_modifier_add_to_infolist, &hook_info_add_to_infolist,
-  &hook_info_hashtable_add_to_infolist, &hook_infolist_add_to_infolist,
-  &hook_hdata_add_to_infolist, &hook_focus_add_to_infolist };
+{
+    &hook_command_add_to_infolist,
+    &hook_command_run_add_to_infolist,
+    &hook_timer_add_to_infolist,
+    &hook_fd_add_to_infolist,
+    &hook_process_add_to_infolist,
+    &hook_connect_add_to_infolist,
+    &hook_line_add_to_infolist,
+    &hook_print_add_to_infolist,
+    &hook_signal_add_to_infolist,
+    &hook_hsignal_add_to_infolist,
+    &hook_config_add_to_infolist,
+    &hook_completion_add_to_infolist,
+    &hook_modifier_add_to_infolist,
+    &hook_info_add_to_infolist,
+    &hook_info_hashtable_add_to_infolist,
+    &hook_infolist_add_to_infolist,
+    &hook_hdata_add_to_infolist,
+    &hook_focus_add_to_infolist,
+    &hook_url_add_to_infolist,
+};
 t_callback_hook *hook_callback_print_log[HOOK_NUM_TYPES] =
-{ &hook_command_print_log, &hook_command_run_print_log,
-  &hook_timer_print_log, &hook_fd_print_log,
-  &hook_process_print_log, &hook_connect_print_log,
-  &hook_line_print_log, &hook_print_print_log,
-  &hook_signal_print_log, &hook_hsignal_print_log,
-  &hook_config_print_log, &hook_completion_print_log,
-  &hook_modifier_print_log, &hook_info_print_log,
-  &hook_info_hashtable_print_log, &hook_infolist_print_log,
-  &hook_hdata_print_log, &hook_focus_print_log };
+{
+    &hook_command_print_log,
+    &hook_command_run_print_log,
+    &hook_timer_print_log,
+    &hook_fd_print_log,
+    &hook_process_print_log,
+    &hook_connect_print_log,
+    &hook_line_print_log,
+    &hook_print_print_log,
+    &hook_signal_print_log,
+    &hook_hsignal_print_log,
+    &hook_config_print_log,
+    &hook_completion_print_log,
+    &hook_modifier_print_log,
+    &hook_info_print_log,
+    &hook_info_hashtable_print_log,
+    &hook_infolist_print_log,
+    &hook_hdata_print_log,
+    &hook_focus_print_log,
+    &hook_url_print_log,
+};
 
 
 /*
@@ -204,8 +255,8 @@ hook_find_pos (struct t_hook *hook)
         {
             if (!ptr_hook->deleted)
             {
-                rc_cmp = string_strcasecmp (HOOK_COMMAND(hook, command),
-                                            HOOK_COMMAND(ptr_hook, command));
+                rc_cmp = string_strcmp (HOOK_COMMAND(hook, command),
+                                        HOOK_COMMAND(ptr_hook, command));
                 if (rc_cmp < 0)
                     return ptr_hook;
                 if ((rc_cmp == 0) && (hook->priority > ptr_hook->priority))
@@ -340,48 +391,6 @@ hook_remove_deleted ()
     }
 }
 
-/*
- * Extracts priority and name from a string.
- *
- * String can be:
- *   - a simple name like "test":
- *       => priority = 1000 (default), name = "test"
- *   - a priority + "|" + name, like "500|test":
- *       => priority = 500, name = "test"
- */
-
-void
-hook_get_priority_and_name (const char *string,
-                            int *priority, const char **name)
-{
-    char *pos, *str_priority, *error;
-    long number;
-
-    if (priority)
-        *priority = HOOK_PRIORITY_DEFAULT;
-    if (name)
-        *name = string;
-
-    pos = strchr (string, '|');
-    if (pos)
-    {
-        str_priority = string_strndup (string, pos - string);
-        if (str_priority)
-        {
-            error = NULL;
-            number = strtol (str_priority, &error, 10);
-            if (error && !error[0])
-            {
-                if (priority)
-                    *priority = number;
-                if (name)
-                    *name = pos + 1;
-            }
-            free (str_priority);
-        }
-    }
-}
-
 /*
  * Initializes a new hook with default values.
  */
@@ -468,6 +477,67 @@ hook_exec_end ()
         hook_remove_deleted ();
 }
 
+/*
+ * Starts execution of a hook callback.
+ */
+
+void
+hook_callback_start (struct t_hook *hook, struct t_hook_exec_cb *hook_exec_cb)
+{
+    if (hook->type == HOOK_TYPE_COMMAND)
+        hook->running++;
+    else
+        hook->running = 1;
+
+    if (debug_long_callbacks > 0)
+    {
+        gettimeofday (&hook_exec_cb->start_time, NULL);
+    }
+    else
+    {
+        hook_exec_cb->start_time.tv_sec = 0;
+        hook_exec_cb->start_time.tv_usec = 0;
+    }
+}
+
+/*
+ * Ends execution of a hook callback.
+ */
+
+void
+hook_callback_end (struct t_hook *hook, struct t_hook_exec_cb *hook_exec_cb)
+{
+    struct timeval end_time;
+    long long time_diff;
+    char *str_diff;
+
+    if (hook->type == HOOK_TYPE_COMMAND)
+        hook->running--;
+    else
+        hook->running = 0;
+
+    if ((debug_long_callbacks > 0)
+        && (hook_exec_cb->start_time.tv_sec > 0))
+    {
+        gettimeofday (&end_time, NULL);
+        time_diff = util_timeval_diff (&hook_exec_cb->start_time, &end_time);
+        if (time_diff >= debug_long_callbacks)
+        {
+            str_diff = util_get_microseconds_string (time_diff);
+            log_printf (
+                _("debug: long callback: hook %s (%s), plugin: %s, "
+                  "subplugin: %s, time elapsed: %s"),
+                hook_type_string[hook->type],
+                hook_get_description (hook),
+                plugin_get_name (hook->plugin),
+                (hook->subplugin) ? hook->subplugin : "-",
+                str_diff);
+            if (str_diff)
+                free (str_diff);
+        }
+    }
+}
+
 /*
  * Returns description of hook.
  *
@@ -496,13 +566,16 @@ hook_set (struct t_hook *hook, const char *property, const char *value)
     if (!hook_valid (hook))
         return;
 
-    if (string_strcasecmp (property, "subplugin") == 0)
+    if (!property)
+        return;
+
+    if (strcmp (property, "subplugin") == 0)
     {
         if (hook->subplugin)
             free (hook->subplugin);
         hook->subplugin = strdup (value);
     }
-    else if (string_strcasecmp (property, "stdin") == 0)
+    else if (strcmp (property, "stdin") == 0)
     {
         if (!hook->deleted
             && (hook->type == HOOK_TYPE_PROCESS)
@@ -514,7 +587,7 @@ hook_set (struct t_hook *hook, const char *property, const char *value)
             (void) num_written;
         }
     }
-    else if (string_strcasecmp (property, "stdin_close") == 0)
+    else if (strcmp (property, "stdin_close") == 0)
     {
         if (!hook->deleted
             && (hook->type == HOOK_TYPE_PROCESS)
@@ -525,7 +598,7 @@ hook_set (struct t_hook *hook, const char *property, const char *value)
             HOOK_PROCESS(hook, child_write[HOOK_PROCESS_STDIN]) = -1;
         }
     }
-    else if (string_strcasecmp (property, "signal") == 0)
+    else if (strcmp (property, "signal") == 0)
     {
         if (!hook->deleted
             && (hook->type == HOOK_TYPE_PROCESS)
@@ -555,6 +628,35 @@ hook_set (struct t_hook *hook, const char *property, const char *value)
     }
 }
 
+/*
+ * Callback used to clean all children (forked processes) by acknowledging
+ * their end.
+ */
+
+int
+hook_timer_clean_children_cb (const void *pointer, void *data,
+                              int remaining_calls)
+{
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) remaining_calls;
+
+    sys_waitpid ();
+
+    return WEECHAT_RC_OK;
+}
+
+/*
+ * Schedule a cleanup timer to clean children (forked processes).
+ */
+
+void
+hook_schedule_clean_children ()
+{
+    hook_timer (NULL, 100, 0, 1, &hook_timer_clean_children_cb, NULL, NULL);
+}
+
 /*
  * Unhooks something.
  */
@@ -734,7 +836,7 @@ hook_add_to_infolist_type (struct t_infolist *infolist, int type,
             switch (ptr_hook->type)
             {
                 case HOOK_TYPE_COMMAND:
-                    match = string_match (HOOK_COMMAND(ptr_hook, command), arguments, 0);
+                    match = string_match (HOOK_COMMAND(ptr_hook, command), arguments, 1);
                     break;
                 default:
                     break;
