@@ -139,9 +139,22 @@ irc_nick_is_nick (struct t_irc_server *server, const char *string)
  */
 
 char *
-irc_nick_find_color (const char *nickname)
+irc_nick_find_color (struct t_irc_server *server, const char *nickname)
 {
-    return weechat_info_get ("nick_color", nickname);
+    char str_args[4096];
+    int casemapping, range;
+
+    casemapping = (server) ? server->casemapping : -1;
+    if ((casemapping < 0) || (casemapping >= IRC_SERVER_NUM_CASEMAPPING))
+        casemapping = IRC_SERVER_CASEMAPPING_RFC1459;
+    range = irc_server_casemapping_range[casemapping];
+
+    snprintf (str_args, sizeof (str_args),
+              "%s;%d",
+              (nickname) ? nickname : "",
+              range);
+
+    return weechat_info_get ("nick_color_ignore_case", str_args);
 }
 
 /*
@@ -151,9 +164,22 @@ irc_nick_find_color (const char *nickname)
  */
 
 char *
-irc_nick_find_color_name (const char *nickname)
+irc_nick_find_color_name (struct t_irc_server *server, const char *nickname)
 {
-    return weechat_info_get ("nick_color_name", nickname);
+    char str_args[4096];
+    int casemapping, range;
+
+    casemapping = (server) ? server->casemapping : -1;
+    if ((casemapping < 0) || (casemapping >= IRC_SERVER_NUM_CASEMAPPING))
+        casemapping = IRC_SERVER_CASEMAPPING_RFC1459;
+    range = irc_server_casemapping_range[casemapping];
+
+    snprintf (str_args, sizeof (str_args),
+              "%s;%d",
+              (nickname) ? nickname: "",
+              range);
+
+    return weechat_info_get ("nick_color_name_ignore_case", str_args);
 }
 
 /*
@@ -256,16 +282,15 @@ irc_nick_set_host (struct t_irc_nick *nick, const char *host)
 }
 
 /*
- * Checks if nick is "op" (or better than "op", for example channel admin or
- * channel owner).
+ * Checks if nick is "op" or higher than "op", like channel admin/owner.
  *
  * Returns:
- *   1: nick is "op" (or better)
+ *   1: nick is "op" or higher
  *   0: nick is not op
  */
 
 int
-irc_nick_is_op (struct t_irc_server *server, struct t_irc_nick *nick)
+irc_nick_is_op_or_higher (struct t_irc_server *server, struct t_irc_nick *nick)
 {
     int index;
 
@@ -397,7 +422,7 @@ irc_nick_get_color_for_nicklist (struct t_irc_server *server,
         if (irc_server_strcasecmp (server, nick->name, server->nick) == 0)
             return strdup (nick_color_self);
         else
-            return irc_nick_find_color_name (nick->name);
+            return irc_nick_find_color_name (server, nick->name);
     }
 
     return strdup (nick_color_bar_fg);
@@ -595,7 +620,7 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     if (irc_server_strcasecmp (server, new_nick->name, server->nick) == 0)
         new_nick->color = strdup (IRC_COLOR_CHAT_NICK_SELF);
     else
-        new_nick->color = irc_nick_find_color (new_nick->name);
+        new_nick->color = irc_nick_find_color (server, new_nick->name);
 
     /* add nick to end of list */
     new_nick->prev_nick = channel->last_nick;
@@ -644,7 +669,7 @@ irc_nick_change (struct t_irc_server *server, struct t_irc_channel *channel,
     if (nick_is_me)
         nick->color = strdup (IRC_COLOR_CHAT_NICK_SELF);
     else
-        nick->color = irc_nick_find_color (nick->name);
+        nick->color = irc_nick_find_color (server, nick->name);
 
     /* add nick in nicklist */
     irc_nick_nicklist_add (server, channel, nick);
@@ -834,44 +859,66 @@ irc_nick_search (struct t_irc_server *server, struct t_irc_channel *channel,
 }
 
 /*
- * Returns number of nicks (total, ops, halfops, voiced, regular) on a channel.
+ * Returns number of nicks per mode on a channel, as an array of integers
+ * whose size is the number of modes + 1 (for regular users).
+ *
+ * For example if modes == "ohv", the array returned has a size of 4, with:
+ *   - array[0] = number of nicks with mode "o"
+ *   - array[1] = number of nicks with mode "h"
+ *   - array[2] = number of nicks with mode "v"
+ *   - array[3] = number of nicks with no mode (regular users)
+ *
+ * The parameter *size is set with the array size (number of integers in the
+ * array, NOT the size in bytes).
+ *
+ * Note: result must be freed after use (if not NULL).
  */
 
-void
+int *
 irc_nick_count (struct t_irc_server *server, struct t_irc_channel *channel,
-                int *total, int *count_ops, int *count_halfops,
-                int *count_voiced, int *count_regular)
+                int *size)
 {
     struct t_irc_nick *ptr_nick;
+    const char *ptr_prefix_modes;
+    int i, *nicks_by_mode, mode_found;
 
-    (*total) = 0;
-    (*count_ops) = 0;
-    (*count_halfops) = 0;
-    (*count_voiced) = 0;
-    (*count_regular) = 0;
-    for (ptr_nick = channel->nicks; ptr_nick;
-         ptr_nick = ptr_nick->next_nick)
+    if (!server || !channel || !size)
+        return NULL;
+
+    *size = 0;
+
+    ptr_prefix_modes = irc_server_get_prefix_modes (server);
+    if (!ptr_prefix_modes)
+        return NULL;
+
+    *size = strlen (ptr_prefix_modes) + 1;
+    nicks_by_mode = (int *)calloc (*size, sizeof (*nicks_by_mode));
+    if (!nicks_by_mode)
     {
-        (*total)++;
-        if (irc_nick_is_op (server, ptr_nick))
+        *size = 0;
+        return NULL;
+    }
+
+    for (ptr_nick = channel->nicks; ptr_nick; ptr_nick = ptr_nick->next_nick)
+    {
+        mode_found = 0;
+        for (i = 0; ptr_prefix_modes[i]; i++)
         {
-            (*count_ops)++;
+            if (irc_nick_has_prefix_mode (server, ptr_nick, ptr_prefix_modes[i]))
+            {
+                nicks_by_mode[i]++;
+                mode_found = 1;
+                break;
+            }
         }
-        else
+        if (!mode_found)
         {
-            if (irc_nick_has_prefix_mode (server, ptr_nick, 'h'))
-            {
-                (*count_halfops)++;
-            }
-            else
-            {
-                if (irc_nick_has_prefix_mode (server, ptr_nick, 'v'))
-                    (*count_voiced)++;
-                else
-                    (*count_regular)++;
-            }
+            /* regular user */
+            nicks_by_mode[*size - 1]++;
         }
     }
+
+    return nicks_by_mode;
 }
 
 /*
@@ -913,7 +960,7 @@ irc_nick_mode_for_display (struct t_irc_server *server, struct t_irc_nick *nick,
     str_prefix[0] = (nick) ? nick->prefix[0] : '\0';
     str_prefix[1] = '\0';
 
-    nick_mode = weechat_config_integer (irc_config_look_nick_mode);
+    nick_mode = weechat_config_enum (irc_config_look_nick_mode);
     if ((nick_mode == IRC_CONFIG_LOOK_NICK_MODE_BOTH)
         || (prefix && (nick_mode == IRC_CONFIG_LOOK_NICK_MODE_PREFIX))
         || (!prefix && (nick_mode == IRC_CONFIG_LOOK_NICK_MODE_ACTION)))
@@ -964,7 +1011,7 @@ irc_nick_as_prefix (struct t_irc_server *server, struct t_irc_nick *nick,
     else if (nick)
         color = strdup (nick->color);
     else if (nickname)
-        color = irc_nick_find_color (nickname);
+        color = irc_nick_find_color (server, nickname);
     else
         color = strdup (IRC_COLOR_CHAT_NICK);
 
@@ -1007,7 +1054,7 @@ irc_nick_color_for_msg (struct t_irc_server *server, int server_message,
         {
             return IRC_COLOR_CHAT_NICK_SELF;
         }
-        color_found = irc_nick_find_color (nickname);
+        color_found = irc_nick_find_color (server, nickname);
         index_color = (index_color + 1) % 16;
         snprintf (color[index_color], sizeof (color[index_color]),
                   "%s",
@@ -1025,12 +1072,13 @@ irc_nick_color_for_msg (struct t_irc_server *server, int server_message,
  */
 
 const char *
-irc_nick_color_for_pv (struct t_irc_channel *channel, const char *nickname)
+irc_nick_color_for_pv (struct t_irc_server *server,
+                       struct t_irc_channel *channel, const char *nickname)
 {
     if (weechat_config_boolean (irc_config_look_color_pv_nick_like_channel))
     {
         if (!channel->pv_remote_nick_color)
-            channel->pv_remote_nick_color = irc_nick_find_color (nickname);
+            channel->pv_remote_nick_color = irc_nick_find_color (server, nickname);
         if (channel->pv_remote_nick_color)
             return channel->pv_remote_nick_color;
     }
