@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,11 +14,11 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_RELAY_UPGRADE_H
-#define WEECHAT_RELAY_UPGRADE_H 1
+#ifndef WEECHAT_PLUGIN_RELAY_UPGRADE_H
+#define WEECHAT_PLUGIN_RELAY_UPGRADE_H
 
 #define RELAY_UPGRADE_FILENAME "relay"
 
@@ -31,7 +31,7 @@ enum t_relay_upgrade_type
     RELAY_UPGRADE_TYPE_SERVER,
 };
 
-extern int relay_upgrade_save ();
+extern int relay_upgrade_save (int force_disconnected_state);
 extern int relay_upgrade_load ();
 
-#endif /* WEECHAT_RELAY_UPGRADE_H */
+#endif /* WEECHAT_PLUGIN_RELAY_UPGRADE_H */
