@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,13 +14,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef WEECHAT_XFER_DCC_H
-#define WEECHAT_XFER_DCC_H 1
+#ifndef WEECHAT_PLUGIN_XFER_DCC_H
+#define WEECHAT_PLUGIN_XFER_DCC_H
 
 extern void xfer_dcc_send_file_child (struct t_xfer *xfer);
 extern void xfer_dcc_recv_file_child (struct t_xfer *xfer);
 
-#endif /* WEECHAT_XFER_DCC_H */
+#endif /* WEECHAT_PLUGIN_XFER_DCC_H */
